@@ -39,7 +39,8 @@ derived_probe::derived_probe (probe *p):
 {
   if (p)
     {
-      this->locations = p->locations;  
+      this->locations = p->locations;
+      this->condition = deep_copy_visitor::deep_copy(p->condition);
       this->tok = p->tok;
       this->privileged = p->privileged;
       this->body = deep_copy_visitor::deep_copy(p->body);
@@ -50,17 +51,42 @@ derived_probe::derived_probe (probe *p):
 derived_probe::derived_probe (probe *p, probe_point *l):
   base (p)
 {
-  if (l)
-    this->locations.push_back (l);
-
   if (p)
     {
+      if (p->condition) 
+        this->condition = deep_copy_visitor::deep_copy(p->condition);
       this->tok = p->tok;
       this->privileged = p->privileged;
       this->body = deep_copy_visitor::deep_copy(p->body);
     }
+
+  if (l)
+    {
+      probe_point *pp = new probe_point (l->components, l->tok);
+      this->locations.push_back (pp);
+      this->add_condition (l->condition);
+      this->insert_condition_statement ();
+    }
 }
 
+void
+derived_probe::insert_condition_statement (void)
+{
+  if (this->condition)
+    {
+      if_statement *ifs = new if_statement ();
+      ifs->tok = this->tok;
+      ifs->thenblock = new next_statement ();
+      ifs->thenblock->tok = this->tok;
+      ifs->elseblock = NULL;
+      unary_expression *notex = new unary_expression ();
+      notex->op = "!";
+      notex->tok = this->tok;
+      notex->operand = this->condition;
+      ifs->condition = notex;
+      body->statements.insert (body->statements.begin(), ifs);
+    }
+}
 
 void
 derived_probe::printsig (ostream& o) const
@@ -331,16 +357,35 @@ match_node::find_and_build (systemtap_session& s,
 
 	  if (match.globmatch(subkey))
 	    {
-	      // recurse
+	      if (s.verbose > 2)
+		clog << "wildcard '" << loc->components[pos]->functor
+		     << "' matched '" << subkey.name << "'" << endl;
+
+	      // When we have a wildcard, we need to create a copy of
+	      // the probe point.  Then we'll create a copy of the
+	      // wildcard component, and substitute the non-wildcard
+	      // functor.
+	      probe_point *non_wildcard_pp = new probe_point(*loc);
+	      probe_point::component *non_wildcard_component
+		= new probe_point::component(*loc->components[pos]);
+	      non_wildcard_component->functor = subkey.name;
+	      non_wildcard_pp->components[pos] = non_wildcard_component;
+
+	      // recurse (with the non-wildcard probe point)
 	      try
 	        {
-		  subnode->find_and_build (s, p, loc, pos+1, results);
+		  subnode->find_and_build (s, p, non_wildcard_pp, pos+1,
+					   results);
 	        }
 	      catch (const semantic_error& e)
 	        {
 		  // Ignore semantic_errors while expanding wildcards.
 		  // If we get done and nothing was expanded, the code
 		  // following the loop will complain.
+
+		  // If this wildcard didn't match, cleanup.
+		  delete non_wildcard_pp;
+		  delete non_wildcard_component;
 		}
 	    }
 	}
@@ -435,6 +480,8 @@ alias_expansion_builder
     // the token location of the alias,
     n->tok = location->tok;
     n->body->tok = location->tok;
+    // The new probe takes over condition.
+    n->add_condition (location->condition);
 
     // and statements representing the concatenation of the alias'
     // body with the use's. 
@@ -553,23 +600,40 @@ derive_probes (systemtap_session& s,
     {
       probe_point *loc = p->locations[i];
 
-      // Pass down optional flag from e.g. alias reference to each
-      // probe_point instance.  We do this by temporarily overriding
-      // the probe_point optional flag.  We could instead deep-copy
-      // and set a flag on the copy permanently.
-
-      bool old_loc_opt = loc->optional;
-      loc->optional = loc->optional || optional;
-
       try
         {
           unsigned num_atbegin = dps.size();
-          s.pattern_root->find_and_build (s, p, loc, 0, dps);
+
+          // Pass down optional flag from e.g. alias reference to each
+          // probe_point instance.  We do this by temporarily overriding
+          // the probe_point optional flag.  We could instead deep-copy
+          // and set a flag on the copy permanently.
+          bool old_loc_opt = loc->optional;
+          loc->optional = loc->optional || optional;
+          s.pattern_root->find_and_build (s, p, loc, 0, dps); // <-- actual derivation!
+          loc->optional = old_loc_opt;
           unsigned num_atend = dps.size();
-          
-          if (! loc->optional && // something required, but
+
+          if (! (loc->optional||optional) && // something required, but
               num_atbegin == num_atend) // nothing new derived!
-            throw semantic_error ("no match for probe point");
+            throw semantic_error ("no match");
+
+          if (loc->sufficient && (num_atend > num_atbegin))
+            {
+              if (s.verbose > 1)
+                {
+                  clog << "Probe point ";
+                  p->locations[i]->print(clog);
+                  clog << " sufficient, skipped";
+                  for (unsigned j = i+1; j < p->locations.size(); ++j)
+                    {
+                      clog << " ";
+                      p->locations[j]->print(clog);
+                    }
+                  clog << endl;
+                }
+              break; // we need not try to derive for any other locations
+            }
         }
       catch (const semantic_error& e)
         {
@@ -584,7 +648,6 @@ derive_probes (systemtap_session& s,
           delete er;
         }
 
-      loc->optional = old_loc_opt;
     }
 }
 
@@ -831,8 +894,7 @@ struct stat_decl_collector
       {
 	assert (e->htype == hist_log);
 	new_stat.type = statistic_decl::logarithmic;
-	assert (e->params.size() == 1);
-	new_stat.logarithmic_buckets = e->params[0];
+	assert (e->params.size() == 0);
       }
 
     map<string, statistic_decl>::iterator i = session.stat_decls.find(sym->name);
@@ -1027,6 +1089,7 @@ systemtap_session::systemtap_session ():
   mark_derived_probes(0), 
   hrtimer_derived_probes(0), 
   perfmon_derived_probes(0), 
+  procfs_derived_probes(0), 
   op (0), up (0)
 {
 }
@@ -1372,7 +1435,10 @@ void semantic_pass_opt1 (systemtap_session& s, bool& relaxed_p)
     {
       if (ftv.traversed.find(s.functions[i]) == ftv.traversed.end())
         {
-          if (s.verbose>2)
+          if (s.functions[i]->tok->location.file == s.user_file->name && // !tapset
+              ! s.suppress_warnings)
+            clog << "WARNING: eliding unused function " << *s.functions[i]->tok << endl;
+          else if (s.verbose>2)
             clog << "Eliding unused function " << s.functions[i]->name
                  << endl;
 	  if (s.tapset_compile_coverage) {
@@ -1412,10 +1478,14 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
     for (unsigned j=0; j<s.probes[i]->locals.size(); /* see below */)
       {
         vardecl* l = s.probes[i]->locals[j];
+
         if (vut.read.find (l) == vut.read.end() &&
             vut.written.find (l) == vut.written.end())
           {
-            if (s.verbose>2)
+            if (l->tok->location.file == s.user_file->name && // !tapset
+                ! s.suppress_warnings)
+              clog << "WARNING: eliding unused variable " << *l->tok << endl;
+            else if (s.verbose>2)
               clog << "Eliding unused local variable "
                    << l->name << " in " << s.probes[i]->name << endl;
 	    if (s.tapset_compile_coverage) {
@@ -1436,7 +1506,10 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
         if (vut.read.find (l) == vut.read.end() &&
             vut.written.find (l) == vut.written.end())
           {
-            if (s.verbose>2)
+            if (l->tok->location.file == s.user_file->name && // !tapset
+                ! s.suppress_warnings)
+              clog << "WARNING: eliding unused variable " << *l->tok << endl;
+            else if (s.verbose>2)
               clog << "Eliding unused local variable "
                    << l->name << " in function " << s.functions[i]->name
                    << endl;
@@ -1457,7 +1530,10 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
       if (vut.read.find (l) == vut.read.end() &&
           vut.written.find (l) == vut.written.end())
         {
-          if (s.verbose>2)
+          if (l->tok->location.file == s.user_file->name && // !tapset
+              ! s.suppress_warnings)
+            clog << "WARNING: eliding unused variable " << *l->tok << endl;
+          else if (s.verbose>2)
             clog << "Eliding unused global variable "
                  << l->name << endl;
 	  if (s.tapset_compile_coverage) {
@@ -1529,9 +1605,17 @@ dead_assignment_remover::visit_assignment (assignment* e)
           e->left->visit (& vut);
           if (vut.side_effect_free ()) // XXX: use _wrt() once we track focal_vars
             {
+              /* PR 1119: NB: This is not necessary here.  A write-only
+                 variable will also be elided soon at the next _opt2 iteration.
+              if (e->left->tok->location.file == session.user_file->name && // !tapset
+                  ! session.suppress_warnings)
+                clog << "WARNING: eliding write-only " << *e->left->tok << endl;
+              else
+              */
               if (session.verbose>2)
                 clog << "Eliding assignment to " << leftvar->name 
                      << " at " << *e->tok << endl;
+              
               *current_expr = e->right; // goodbye assignment*
               relaxed_p = false;
             }
@@ -1577,6 +1661,9 @@ struct dead_stmtexpr_remover: public traversing_visitor
     session(s), relaxed_p(r), current_stmt(0) {}
 
   void visit_block (block *s);
+  void visit_if_statement (if_statement* s);
+  void visit_foreach_loop (foreach_loop *s);
+  void visit_for_loop (for_loop *s);
   // XXX: and other places where stmt_expr's might be nested
 
   void visit_expr_statement (expr_statement *s);
@@ -1595,6 +1682,39 @@ dead_stmtexpr_remover::visit_block (block *s)
     }
 }
 
+void
+dead_stmtexpr_remover::visit_if_statement (if_statement *s)
+{
+  statement** last_stmt = current_stmt;
+  current_stmt = & s->thenblock;
+  s->thenblock->visit (this);
+  if (s->elseblock)
+    {
+      current_stmt = & s->elseblock;
+      s->elseblock->visit (this);
+    }
+  current_stmt = last_stmt;
+}
+
+void
+dead_stmtexpr_remover::visit_foreach_loop (foreach_loop *s)
+{
+  statement** last_stmt = current_stmt;
+  current_stmt = & s->block;
+  s->block->visit (this);
+  current_stmt = last_stmt;
+}
+
+void
+dead_stmtexpr_remover::visit_for_loop (for_loop *s)
+{
+  statement** last_stmt = current_stmt;
+  current_stmt = & s->block;
+  s->block->visit (this);
+  current_stmt = last_stmt;
+}
+
+
 
 void
 dead_stmtexpr_remover::visit_expr_statement (expr_statement *s)
@@ -1614,8 +1734,16 @@ dead_stmtexpr_remover::visit_expr_statement (expr_statement *s)
 
   varuse_collecting_visitor vut;
   s->value->visit (& vut);
-  if (vut.side_effect_free_wrt (focal_vars))
+  if (vut.side_effect_free_wrt (focal_vars) &&
+      *current_stmt == s) // we're not nested any deeper than expected 
     {
+      /* PR 1119: NB: this message is not a good idea here.  It can
+         name some arbitrary RHS expression of an assignment.
+      if (s->value->tok->location.file == session.user_file->name && // not tapset
+          ! session.suppress_warnings)
+        clog << "WARNING: eliding read-only " << *s->value->tok << endl;
+      else 
+      */
       if (session.verbose>2)
         clog << "Eliding side-effect-free expression "
              << *s->tok << endl;
@@ -1783,10 +1911,7 @@ semantic_pass_optimize1 (systemtap_session& s)
     }
 
   if (s.probes.size() == 0)
-    {
-      cerr << "semantic error: no probes found." << endl;
-      rc = 1;
-    }
+    throw semantic_error ("no probes found");
 
   return rc;
 }
@@ -1810,10 +1935,7 @@ semantic_pass_optimize2 (systemtap_session& s)
     }
 
   if (s.probes.size() == 0)
-    {
-      cerr << "semantic error: no probes found." << endl;
-      rc = 1;
-    }
+    throw semantic_error ("no probes found");
 
   return rc;
 }
@@ -1838,7 +1960,6 @@ semantic_pass_types (systemtap_session& s)
   while (1)
     {
       iterations ++;
-      // cerr << "Type resolution, iteration " << iterations << endl;
       ti.num_newly_resolved = 0;
       ti.num_still_unresolved = 0;
 
@@ -2729,15 +2850,12 @@ typeresolution_info::unresolved (const token* tok)
 
   if (assert_resolvability)
     {
+      stringstream msg;
       string nm = (current_function ? current_function->name :
                    current_probe ? current_probe->name :
                    "?");
-      cerr << "semantic error: " + nm + " with unresolved type for ";
-      if (tok)
-        cerr << *tok;
-      else
-        cerr << "a token";
-      cerr << endl;
+      msg << nm + " with unresolved type";
+      session.print_error (semantic_error (msg.str(), tok));
     }
 }
 
@@ -2749,22 +2867,15 @@ typeresolution_info::invalid (const token* tok, exp_type pe)
 
   if (assert_resolvability)
     {
+      stringstream msg;
       string nm = (current_function ? current_function->name :
                    current_probe ? current_probe->name :
                    "?");
       if (tok && tok->type == tok_operator)
-        {
-	  cerr << "semantic error: " + nm + " uses invalid " << *tok;
-        }
+        msg << nm + " uses invalid operator";
       else
-        {
-	  cerr << "semantic error: " + nm + " with invalid type " << pe << " for ";
-	  if (tok)
-	      cerr << *tok;
-	  else
-	      cerr << "a token";
-	}
-      cerr << endl;
+        msg << nm + " with invalid type " << pe;
+      session.print_error (semantic_error (msg.str(), tok));
     }
 }
 
@@ -2776,15 +2887,12 @@ typeresolution_info::mismatch (const token* tok, exp_type t1, exp_type t2)
 
   if (assert_resolvability)
     {
+      stringstream msg;
       string nm = (current_function ? current_function->name :
                    current_probe ? current_probe->name :
                    "?");
-      cerr << "semantic error: " + nm + " with type mismatch for ";
-      if (tok)
-        cerr << *tok;
-      else
-        cerr << "a token";
-      cerr << ": " << t1 << " vs. " << t2 << endl;
+      msg << nm + " with type mismatch (" << t1 << " vs. " << t2 << ")";
+      session.print_error (semantic_error (msg.str(), tok));
     }
 }
 
@@ -2793,6 +2901,5 @@ void
 typeresolution_info::resolved (const token*, exp_type)
 {
   num_newly_resolved ++;
-  // cerr << "resolved " << *e->tok << " type " << t << endl;
 }
 
