@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * I/O for printing warnings, errors and debug messages
- * Copyright (C) 2005, 2006 Red Hat Inc.
+ * Copyright (C) 2005, 2006, 2007 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -11,11 +11,6 @@
 #ifndef _IO_C_
 #define _IO_C_
 
-#include "transport/transport.c"
-
-void _stp_print_flush (void);
-void _stp_string_cat_cstr (String str1, const char *str2);
-
 /** @file io.c
  * @brief I/O for printing warnings, errors and debug messages.
  */
@@ -23,23 +18,18 @@ void _stp_string_cat_cstr (String str1, const char *str2);
  * @{
  */
 
-/** private buffer for _stp_log() */
-#define STP_LOG_BUF_LEN 2047
 #define WARN_STRING "WARNING: "
 #define ERR_STRING "ERROR: "
-
-static char _stp_lbuf[NR_CPUS][STP_LOG_BUF_LEN + 1];
-
 enum code { INFO=0, WARN, ERROR, DBUG };
 
-static void _stp_vlog (enum code type, char *func, int line, const char *fmt, va_list args)
+static void _stp_vlog (enum code type, const char *func, int line, const char *fmt, va_list args)
 {
 	int num;
-	char *buf = &_stp_lbuf[get_cpu()][0];
+	char *buf = per_cpu_ptr(Stp_lbuf, smp_processor_id());
 	int start = 0;
 
 	if (type == DBUG) {
-		start = scnprintf(buf, STP_LOG_BUF_LEN, "\033[36m%s:%d:\033[0m ", func, line);
+		start = _stp_snprintf(buf, STP_LOG_BUF_LEN, "\033[36m%s:%d:\033[0m ", func, line);
 	} else if (type == WARN) {
 		strcpy (buf, WARN_STRING);
 		start = sizeof(WARN_STRING) - 1;
@@ -48,33 +38,25 @@ static void _stp_vlog (enum code type, char *func, int line, const char *fmt, va
 		start = sizeof(ERR_STRING) - 1;
 	}
 
-	num = vscnprintf (buf + start, STP_LOG_BUF_LEN - start, fmt, args);
+	num = _stp_vscnprintf (buf + start, STP_LOG_BUF_LEN - start - 1, fmt, args);
 	if (num + start) {
 		if (buf[num + start - 1] != '\n') {
 			buf[num + start] = '\n';
 			num++;
+			buf[num + start] = '\0';
 		}
-		buf[num + start] = '\0';
-		
-#ifdef STP_RELAYFS
+
 		if (type != DBUG)
-			_stp_write(STP_OOB_DATA, buf, start + num + 1);
-		_stp_string_cat_cstr(_stp_stdout,buf);
-		_stp_print_flush();
-#else
-		if (type != DBUG)
-			_stp_write(STP_OOB_DATA, buf, start + num + 1);
+			_stp_ctl_write(STP_OOB_DATA, buf, start + num + 1);
 		else {
-			_stp_string_cat_cstr(_stp_stdout,buf);
+			_stp_print(buf);
 			_stp_print_flush();
 		}
-#endif
 	}
-	put_cpu();
 }
 
 /** Logs Data.
- * This function sends the message immediately to stpd. It
+ * This function sends the message immediately to staprun. It
  * will also be sent over the bulk transport (relayfs) if it is
  * being used. If the last character is not a newline, then one 
  * is added. This function is not as efficient as _stp_printf()
@@ -92,7 +74,7 @@ void _stp_log (const char *fmt, ...)
 }
 
 /** Prints warning.
- * This function sends a warning message immediately to stpd. It
+ * This function sends a warning message immediately to staprun. It
  * will also be sent over the bulk transport (relayfs) if it is
  * being used. If the last character is not a newline, then one 
  * is added. 
@@ -107,11 +89,11 @@ void _stp_warn (const char *fmt, ...)
 }
 
 /** Exits and unloads the module.
- * This function sends a signal to stpd to tell it to
+ * This function sends a signal to staprun to tell it to
  * unload the module and exit. The module will not be 
  * unloaded until after the current probe returns.
  * @note Be careful to not treat this like the Linux exit() 
- * call. You should probably call return immediately after
+ * call. You should probably call return immediately after 
  * calling _stp_exit().
  */
 void _stp_exit (void)
@@ -120,7 +102,7 @@ void _stp_exit (void)
 }
 
 /** Prints error message and exits.
- * This function sends an error message immediately to stpd. It
+ * This function sends an error message immediately to staprun. It
  * will also be sent over the bulk transport (relayfs) if it is
  * being used. If the last character is not a newline, then one 
  * is added. 
@@ -140,7 +122,7 @@ void _stp_error (const char *fmt, ...)
 
 
 /** Prints error message.
- * This function sends an error message immediately to stpd. It
+ * This function sends an error message immediately to staprun. It
  * will also be sent over the bulk transport (relayfs) if it is
  * being used. If the last character is not a newline, then one 
  * is added. 
@@ -157,7 +139,7 @@ void _stp_softerror (const char *fmt, ...)
 }
 
 
-static void _stp_dbug (char *func, int line, const char *fmt, ...)
+static void _stp_dbug (const char *func, int line, const char *fmt, ...)
 {
 	va_list args;
 	va_start(args, fmt);
