@@ -85,14 +85,15 @@ Stat _stp_stat_init (int type, ...)
 		va_start (ap, type);
 		
 		if (type == HIST_LOG) {
-			buckets = va_arg(ap, int);
+			buckets = HIST_LOG_BUCKETS;
 		} else {
 			start = va_arg(ap, int);
 			stop = va_arg(ap, int);
 			interval = va_arg(ap, int);
-			/* FIXME. check interval != 0 and not too large */
-			buckets = (stop - start) / interval;
-			if ((stop - start) % interval) buckets++;
+
+			buckets = _stp_stat_calc_buckets(stop, start, interval);
+			if (!buckets)
+				return NULL;
 		}
 		va_end (ap);
 	}
