@@ -35,6 +35,7 @@ struct profile_derived_probe_group;
 struct mark_derived_probe_group;
 struct hrtimer_derived_probe_group;
 struct perfmon_derived_probe_group;
+struct procfs_derived_probe_group;
 struct embeddedcode;
 struct translator_output;
 struct unparser;
@@ -47,18 +48,15 @@ struct statistic_decl
 {
   statistic_decl()
     : type(none), 
-      logarithmic_buckets(0),
       linear_low(0), linear_high(0), linear_step(0)
   {}    
   enum { none, linear, logarithmic } type;
-  int64_t logarithmic_buckets;
   int64_t linear_low;
   int64_t linear_high;
   int64_t linear_step;
   bool operator==(statistic_decl const & other)
   {
     return type == other.type 
-      && logarithmic_buckets == other.logarithmic_buckets
       && linear_low == other.linear_low
       && linear_high == other.linear_high
       && linear_step == other.linear_step;
@@ -87,17 +85,19 @@ struct systemtap_session
   int target_pid;
   int last_pass;
   unsigned verbose;
-  unsigned timing;
+  bool timing;
   bool keep_tmpdir;
   bool guru_mode;
   bool bulk_mode;
   bool unoptimized;
   bool merge;
+  bool suppress_warnings;
   int buffer_size;
   unsigned perfmon;
   bool symtab; /* true: emit symbol table at translation time; false: let staprun do it. */
   bool prologue_searching;
   bool tapset_compile_coverage;
+  bool need_uprobes;
 
   // Cache data
   bool use_cache;
@@ -139,6 +139,7 @@ struct systemtap_session
   mark_derived_probe_group* mark_derived_probes;
   hrtimer_derived_probe_group* hrtimer_derived_probes;
   perfmon_derived_probe_group* perfmon_derived_probes;
+  procfs_derived_probe_group* procfs_derived_probes;
 
   // unparser data
   translator_output* op;
