@@ -75,19 +75,20 @@ symboldecl::~symboldecl ()
 
 probe_point::probe_point (std::vector<component*> const & comps,
 			  const token * t):
-  components(comps), tok(t), optional (false)
+  components(comps), tok(t), optional (false), sufficient (false),
+  condition (0)
 {
 }
 
 probe_point::probe_point ():
-  tok (0), optional (false)
+  tok (0), optional (false), sufficient (false), condition (0)
 {
 }
 
 
 unsigned probe::last_probeidx = 0;
 probe::probe ():
-  body (0), tok (0)
+  body (0), tok (0), condition (0)
 {
   this->name = string ("probe_") + lex_cast<string>(last_probeidx ++);
 }
@@ -338,6 +339,55 @@ void functioncall::print (ostream& o) const
 }  
 
 
+bool
+print_format::parse_print(const std::string &name,
+  bool &stream, bool &format, bool &delim, bool &newline, bool &_char)
+{
+  const char *n = name.c_str();
+
+  stream = true;
+  format = delim = newline = _char = false;
+
+  if (strcmp(n, "print_char") == 0)
+    {
+      _char = true;
+      return true;
+    }
+
+  if (*n == 's')
+    {
+      stream = false;
+      ++n;
+    }
+
+  if (0 != strncmp(n, "print", 5))
+    return false;
+  n += 5;
+
+  if (*n == 'f')
+    {
+      format = true;
+      ++n;
+    }
+  else
+    {
+      if (*n == 'd')
+        {
+	  delim = true;
+	  ++n;
+	}
+
+      if (*n == 'l' && *(n+1) == 'n')
+        {
+	  newline = true;
+	  n += 2;
+	}
+    }
+
+  return (*n == '\0');
+}
+
+
 string 
 print_format::components_to_string(vector<format_component> const & components)
 {
@@ -481,7 +531,7 @@ print_format::string_to_components(string const & str)
 	break;
 
       // Now we are definitely parsing a conversion. 
-      // Begin by parsing flags (whicih are optional).
+      // Begin by parsing flags (which are optional).
 
       switch (*i)
 	{
@@ -611,20 +661,17 @@ print_format::string_to_components(string const & str)
 
 void print_format::print (ostream& o) const
 {
-  string name = (string(print_to_stream ? "" : "s") 
-		 + string("print") 
-		 + string(print_with_format ? "f" : ""));
-  o << name << "(";
+  o << tok->content << "(";
   if (print_with_format)
-    {
-      o << lex_cast_qstring (raw_components);
-    }
+    o << lex_cast_qstring (raw_components);
+  if (print_with_delim)
+    o << lex_cast_qstring (delimiter.literal_string);
   if (hist)
     hist->print(o);
   for (vector<expression*>::const_iterator i = args.begin();
        i != args.end(); ++i)
     {
-      if (i != args.begin() || print_with_format)
+      if (i != args.begin() || print_with_format || print_with_delim)
 	o << ", ";
       (*i)->print(o);
     }
@@ -678,13 +725,9 @@ hist_op::print (ostream& o) const
       break;
 
     case hist_log:
-      assert(params.size() == 1);
+      assert(params.size() == 0);
       o << "hist_log(";
       stat->print(o);
-      for (size_t i = 0; i < params.size(); ++i)
-	{
-	  o << ", " << params[i];
-	}
       o << ")";
       break;
     }
@@ -843,7 +886,7 @@ void probe::printsig (ostream& o) const
 {
   for (unsigned i=0; i<locations.size(); i++)
     {
-      if (i > 0) o << "," << endl;
+      if (i > 0) o << ",";
       locations[i]->print (o);
     }
 }
@@ -855,6 +898,26 @@ probe::collect_derivation_chain (std::vector<derived_probe*> &probes_list)
   probes_list.push_back((derived_probe*)this);
 }
 
+void
+probe::add_condition (expression* e)
+{
+  if (e)
+    {
+      if (this->condition)
+        {
+          logical_and_expr *la = new logical_and_expr ();
+          la->op = "&&";
+          la->left = this->condition;
+          la->right = e;
+          la->tok = e->tok;
+          this->condition = la;
+        }
+      else
+        {
+          this->condition = e;
+        }
+    }
+}
 
 void probe_point::print (ostream& o) const
 {
@@ -866,8 +929,12 @@ void probe_point::print (ostream& o) const
       if (c->arg)
         o << "(" << *c->arg << ")";
     }
-  if (optional)
+  if (sufficient)
+    o << "!";
+  else if (optional) // sufficient implies optional
     o << "?";
+  if (condition)
+    o<< " if (" << *condition << ")";
 }
 
 string probe_point::str ()
@@ -881,8 +948,12 @@ string probe_point::str ()
       if (c->arg)
         o << "(" << *c->arg << ")";
     }
-  if (optional)
+  if (sufficient)
+    o << "!";
+  else if (optional) // sufficient implies optional
     o << "?";
+  if (condition)
+    o<< " if (" << *condition << ")";
   return o.str();
 }
 
@@ -902,7 +973,7 @@ void probe_alias::printsig (ostream& o) const
   o << " = ";
   for (unsigned i=0; i<locations.size(); i++)
     {
-      o << (i>0 ? ", " : "");
+      if (i > 0) o << ", ";
       locations[i]->print (o);
     }
 }
@@ -2252,10 +2323,14 @@ deep_copy_visitor::visit_print_format (print_format* e)
 {
   print_format* n = new print_format;
   n->tok = e->tok;
-  n->print_with_format = e->print_with_format;
   n->print_to_stream = e->print_to_stream;
+  n->print_with_format = e->print_with_format;
+  n->print_with_delim = e->print_with_delim;
+  n->print_with_newline = e->print_with_newline;
+  n->print_char = e->print_char;
   n->raw_components = e->raw_components;
   n->components = e->components;
+  n->delimiter = e->delimiter;
   for (unsigned i = 0; i < e->args.size(); ++i)
     {
       expression* na;
@@ -2305,3 +2380,12 @@ deep_copy_visitor::deep_copy (statement* s)
   require <statement*> (&v, &n, s);
   return n;
 }
+
+expression*
+deep_copy_visitor::deep_copy (expression* s)
+{
+  expression* n;
+  deep_copy_visitor v;
+  require <expression*> (&v, &n, s);
+  return n;
+}
