@@ -262,8 +262,11 @@ struct functioncall: public expression
 
 struct print_format: public expression
 {
-  bool print_with_format;
   bool print_to_stream;
+  bool print_with_format;
+  bool print_with_delim;
+  bool print_with_newline;
+  bool print_char;
 
   enum format_flag
     {
@@ -320,11 +323,14 @@ struct print_format: public expression
 
   std::string raw_components;
   std::vector<format_component> components;
+  format_component delimiter;
   std::vector<expression*> args;
   hist_op *hist;
 
   static std::string components_to_string(std::vector<format_component> const & components);
   static std::vector<format_component> string_to_components(std::string const & str);
+  static bool parse_print(const std::string &name, bool &stream, 
+	            bool &format, bool &delim, bool &newline, bool &_char);
 
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -555,8 +561,6 @@ struct stapfile
 };
 
 
-
-
 struct probe_point
 {
   struct component // XXX: sort of a restricted functioncall
@@ -569,6 +573,8 @@ struct probe_point
   std::vector<component*> components;
   const token* tok; // points to first component's functor
   bool optional;
+  bool sufficient;
+  expression* condition;
   void print (std::ostream& o) const;
   probe_point ();
   probe_point(std::vector<component*> const & comps,const token * t);
@@ -585,8 +591,10 @@ struct probe
   const token* tok;
   std::vector<vardecl*> locals;
   std::vector<vardecl*> unused_locals;
+  expression* condition;
   probe ();
   void print (std::ostream& o) const;
+  void add_condition (expression* e);
   virtual void printsig (std::ostream &o) const;
   virtual void collect_derivation_chain (std::vector<derived_probe*> &probes_list);
   virtual probe* basest () { return this; }
@@ -789,6 +797,7 @@ struct deep_copy_visitor: public visitor
 {
   std::stack<void *> targets;
 
+  static expression *deep_copy (expression *s);
   static statement *deep_copy (statement *s);
   static block *deep_copy (block *s);
 
